package com.bernhardriedl.projects.doublefinder.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;

/**
 * FileDrop 1.1
 * by Robert Harder
 * http://iharder.sourceforge.net/current/java/filedrop/
 */

import net.iharder.dnd.FileDrop;

import com.bernhardriedl.projects.doublefinder.DoubleFinder;
import com.bernhardriedl.projects.doublefinder.images.ImagesContainer;

/**
 * Tools Library 2.10
 * https://www.bernhard-riedl.com/projects/
 */

import com.bernhardriedl.projects.tools.gui.AboutDialog;
import com.bernhardriedl.projects.tools.gui.EnhancedButton;

public class MainMenu
	extends JFrame
	implements ActionListener, FileDrop.Listener {

	private static final long serialVersionUID = -7242707059731572526L;

	private EnhancedButton button_start=new EnhancedButton("Eliminate Duplicates");
	private EnhancedButton button_select=new EnhancedButton("Select File");
	private EnhancedButton button_open=new EnhancedButton();
	private JTextField textfield_file=new JTextField();
	
	private Desktop desktop=null;

	/**
	 * menubar
	 */

	private JMenuBar menubar = new JMenuBar();
	private JMenu menu_file = new JMenu();
	private JMenu menu_help = new JMenu();

	private JMenuItem menuitem_select = new JMenuItem();
	private JCheckBoxMenuItem menuitem_createBackup = new JCheckBoxMenuItem();
	private JMenuItem menuitem_exit = new JMenuItem();

	private JMenuItem menuitem_about = new JMenuItem();

	private JFileChooser chooser=new JFileChooser();

	private JPanel selected_file=new JPanel();
	private JPanel content=new JPanel();

	/**
	 * Constructor
	 */

	public MainMenu() {
		try {
			this.setTitle(DoubleFinder.NICENAME);

			ImagesContainer.loadPics();
			
			if (Desktop.isDesktopSupported()) {
				Desktop tempDesktop = Desktop.getDesktop();

				if (tempDesktop.isSupported(Desktop.Action.OPEN)) {
					this.desktop=tempDesktop;
					}
			}

			this.setIconImage(ImagesContainer.icon.getImage());
			
			this.chooser.setApproveButtonText("Select");
			this.chooser.setDialogTitle("Select File");

			this.makeComponents();
			this.setContentPane(content);

			this.makeMenu();
			this.setJMenuBar(menubar);

			this.getRootPane().setDefaultButton(this.button_start);

			this.setSize(400,150);

			com.bernhardriedl.projects.tools.gui.Utils.centerWindow(this);
			this.setVisible(true);
		}

		catch (Exception e) {
			JOptionPane.showMessageDialog(this, e.getMessage());
		}
	}
	
	/**
	 * Initialize GUI components
	 * @throws Exception
	 */

	private void makeComponents() throws Exception {
		this.textfield_file.setEditable(false);
		this.textfield_file.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

		this.button_open.addActionListener(this);
		this.button_start.addActionListener(this);
		this.button_select.addActionListener(this);

		this.button_start.setEnabled(false);		
		
		this.selected_file.setLayout(new BorderLayout());
		this.selected_file.setBorder(BorderFactory.createLineBorder(Color.GRAY));
		this.selected_file.add(this.textfield_file, BorderLayout.CENTER);

		new FileDrop(this.button_select, this);
		new FileDrop(this.textfield_file, this);
		
		this.button_select.setToolTipText("you can also use drag and drop...");
		this.textfield_file.setToolTipText("you can also use drag and drop...");

		if (this.desktop!=null) {
			this.button_open.setIcon(ImagesContainer.iconEdit16);
			this.button_open.setToolTipText("open selected file");
			this.button_open.setEnabled(false);
			this.selected_file.add(this.button_open, BorderLayout.EAST);
		}

		this.content.setLayout(new GridLayout(3,0));
		this.content.add(this.button_select);
		this.content.add(this.selected_file);
		this.content.add(this.button_start);

		this.setContentPane(this.content);
	}

	/**
	 * Initialize menu
	 * @throws Exception
	 */
	
	private void makeMenu() throws Exception {
		this.menuitem_select.setText("Select File");
		this.menuitem_select.addActionListener(this);
		this.menuitem_select.setIcon(ImagesContainer.iconOpen16);

		this.menuitem_createBackup.setText("Create Backup");
		this.menuitem_createBackup.setSelected(true);
		this.menuitem_createBackup.addActionListener(this);

		this.menuitem_exit.setIcon(ImagesContainer.iconStop16);
		this.menuitem_exit.setText("Exit");
		this.menuitem_exit.addActionListener(this);
		this.menuitem_exit.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.KeyEvent.ALT_MASK, false));

		this.menuitem_about.setIcon(ImagesContainer.iconAbout16);
		this.menuitem_about.setText("About");
		this.menuitem_about.addActionListener(this);
		this.menuitem_about.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F1, 0, false));

		this.menu_file.setText("File");
		this.menu_file.setMnemonic('F');
		this.menu_file.add(this.menuitem_select);
		this.menu_file.addSeparator();
		this.menu_file.add(this.menuitem_createBackup);
		this.menu_file.addSeparator();
		this.menu_file.add(this.menuitem_exit);

		this.menu_help.setText("Help");
		this.menu_help.add(this.menuitem_about);
		this.menu_help.setMnemonic('H');

		this.menubar.add(this.menu_file);
		this.menubar.add(Box.createHorizontalGlue());
		this.menubar.add(this.menu_help);
	}
	
	private void setFile(String file) {
		this.textfield_file.setText(file);
		this.textfield_file.setToolTipText(file);
		this.button_start.setEnabled(true);
		this.button_open.setEnabled(true);
	}
	
	/**
	 * opens select file dialog
	 */
	
	private void selectFile() {
		int return_val=this.chooser.showOpenDialog(this);

		if (return_val==JFileChooser.APPROVE_OPTION)
			this.setFile(chooser.getSelectedFile().getAbsolutePath());
	}
	
	/**
	 * opens the selected File 
	 * @throws IOException
	 */
	
	private void openSelectedFile() {
		try {
			this.desktop.open(new File(this.textfield_file.getText()));
		}
		catch (Exception ex) {
			JOptionPane.showMessageDialog(this, "Sorry, this file could not be opened!");
		}			
	}

	/**
	 * actionPerformed
	 * @param e ActionEvent
	 */

	public void actionPerformed(ActionEvent e) {
		if (e.getSource()==this.button_start) {
			try {
				com.bernhardriedl.projects.doublefinder.logic.DoubleFinder doubleFinder = new com.bernhardriedl.projects.doublefinder.logic.DoubleFinder(textfield_file.getText(), this.menuitem_createBackup.isSelected());

				Vector <Object> values= new Vector <Object>();
				values.addElement("Thanks!");
				
				if(this.desktop!=null) {
					values.addElement("Open File");
				}
				
				int response=JOptionPane.showOptionDialog(this, this.textfield_file.getText()+" is now free of duplicates ("+ doubleFinder.getCounter() +" removed)!", "Completed successfully!", JOptionPane.DEFAULT_OPTION,  JOptionPane.INFORMATION_MESSAGE, null, values.toArray(), values.elementAt(0));
				
				if (response==1)
					this.openSelectedFile();
			}
			catch (Exception ex) {
				JOptionPane.showMessageDialog(this, ex.getMessage(), "An error occured!", JOptionPane.ERROR_MESSAGE);
			}
		}

		else if (e.getSource().equals(this.button_select) || e.getSource().equals(this.menuitem_select)) {
			this.selectFile();
		}
		
		else if (e.getSource().equals(this.button_open)) {
			this.openSelectedFile();
		}

		else if (e.getSource().equals(this.menuitem_exit)) {
			com.bernhardriedl.projects.tools.gui.Utils.exitProgramm(this);
		}

		else if (e.getSource().equals(this.menuitem_about)) {
			Vector <String> text=new Vector <String> ();

			text.addElement("Look for and eliminate duplicate lines in textfiles.");
			text.addElement("I use this thingy to keep songs in my playlists unique.");

			new AboutDialog(this, DoubleFinder.NICENAME, DoubleFinder.VERSION, "2004-2012", "Dr. Bernhard Riedl", text, new Vector <String> (), 420, 140);
		}
	}

	/**
	 * setLookAndFeel
	 */

	static {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}

		catch(Exception ex) {

			/**
			 * do nothing
			 */

		}
	}

	/**
	 * override processWindowEvent
	 * to allow for a 'Are you sure'-prompt
	 * @param e WindowEvent
	 */

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING)
			com.bernhardriedl.projects.tools.gui.Utils.exitProgramm(this);
	}
	
	public void filesDropped(File[] files) {
		if (files.length>0)
			try {
				this.setFile(files[0].getAbsolutePath());
			}
			catch (Exception ex) {
				JOptionPane.showMessageDialog(null, "An error occurred while loading the file!", "Error", JOptionPane.ERROR_MESSAGE);
		}
		
	}
}